import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Line2D;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.AffineTransform;

/**
 * Ein Badewanne, der manipuliert werden kann und sich selbst auf einer Leinwand zeichnet.
 * 
 * @author Claus Albowski
 * @version 2.1  (aug 06)
 */
public class Badewanne
{
  private int xPosition;
  private int yPosition;
  private int orientierung;
  private String farbe;
  private boolean istSichtbar;
  private int breite;
  private int tiefe;


  /**
   * Erzeuge einen neuen Badewanne mit einer Standardfarbe und Standardgroesse
   * an einer Standardposition.
   */
    public Badewanne()
  {
    xPosition = 20;
    yPosition = 20;
    farbe = "blau";
    orientierung = 0;
    istSichtbar = false;
    breite = 180;
    tiefe  = 80;
  }
  
  /**
   * Berechnet das zu zeichnende Shape anhand der gegebenen Daten
   * [ Zum Anzeigen der Attributwerte ber Inspect muss hier die Sichtbarkeit 
   *  auf public gesetzt werden. ]
   */
  private Shape gibAktuelleFigur()
  {
      // definieren
      GeneralPath badewanne = new GeneralPath();
      Rectangle2D umriss = new Rectangle2D.Double(0 , 0, breite, tiefe); //knnen wir gut gebrauchen!
      badewanne.append(umriss, false);

      Line2D obererRand = new Line2D.Double(0.1*tiefe, 0.1*tiefe, breite - 0.5*tiefe, 0.1*tiefe);
      Line2D linkerRand = new Line2D.Double(0.1*tiefe, 0.1*tiefe, 0.1*tiefe, 0.9*tiefe);
      Line2D untererRand = new Line2D.Double(0.1*tiefe, 0.9*tiefe, breite - 0.5*tiefe, 0.9*tiefe);
      Arc2D bogen = new Arc2D.Double(breite - 0.9*tiefe, 0.1*tiefe,  0.8*tiefe, 0.8*tiefe, 270, 180, Arc2D.OPEN);
      badewanne.append(obererRand, false);
      badewanne.append(linkerRand, false);
      badewanne.append(untererRand, false);
      badewanne.append(bogen, false);
      
      Ellipse2D ablauf = new Ellipse2D.Double(0.1*breite, 0.5*tiefe-2, 4, 4);
      badewanne.append(ablauf, false);

      // transformieren
      AffineTransform t = new AffineTransform();
      t.translate(xPosition, yPosition);
      t.rotate(Math.toRadians(orientierung),umriss.getX()+umriss.getWidth()/2,umriss.getY()+umriss.getHeight()/2);
      return  t.createTransformedShape(badewanne);
  }
  
  /**
   * Mache dieses Objekt sichtbar. Wenn es bereits sichtbar ist, tue nichts.
   */
  public void zeige()
  {
    istSichtbar = true;
    zeichne();
  }

  /**
   * Mache dieses Objekt unsichtbar. Wenn es bereits unsichtbar ist, tue nichts.
   */
  public void verberge()
  {
    loesche();
    istSichtbar = false;
  }

  /**
   * Drehe auf den angegebenen Winkel
   */
  public void dreheAuf(int neuerWinkel)
  {
     loesche();
     orientierung = neuerWinkel;
     zeichne();
  }

  /**
   * Bewege dieses Objekt horizontal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeHorizontal(int entfernung)
  {
    loesche();
    xPosition += entfernung;
    zeichne();
  }

  /**
   * Bewege dieses objekt vertikal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeVertikal(int entfernung)
  {
    loesche();
    yPosition += entfernung;
    zeichne();
  }

 
  /**
   * Aendere die Farbe dieses Objektes in 'neueFarbe'.
   * Gueltige Angaben sind "rot", "gelb", "blau", "gruen",
   * "lila" und "schwarz".
   */
  public void aendereFarbe(String neueFarbe)
  {
    farbe = neueFarbe;
    zeichne();
  }

  /**
   * Zeichne dieses Objekt mit seinen aktuellen Werten auf den Bildschirm.
   */
  private void zeichne()
  {
    if (istSichtbar)
    {
      Shape figur = gibAktuelleFigur();
      Leinwand leinwand = Leinwand.gibLeinwand();
      leinwand.zeichne (
        this,           // leinwand kennt das Objekt
        farbe,          // definiert seine Zeichenfarbe
        figur);         // definiert seinen grafischen Aspekt
      leinwand.warte(10);
    }
  }

  /**
   * Loesche dieses Objekt vom Bildschirm.
   */
  private void loesche()
  {
    if (istSichtbar)
    {
      Leinwand leinwand = Leinwand.gibLeinwand();
      leinwand.entferne(this);
    }
  }
}
